package cz.muni.fi.pa165.jdbc;

import java.sql.*;
import java.io.*;

public class Updatable {
	
  private static final int TOTAL_MSGS = 100;	

  static Employee[] employees =
  {
  	new Employee(1, "Pavel", "Novak"),
  	new Employee(2, "Jiri", "Berkovec"),
  	new Employee(3, "Josef", "Makovsky")
  };

  protected static Message newMessage(int i)
  {
  	return new Message(employees[i % employees.length], employees[(i+1) % employees.length], "Zpravicka c."+i);
  }

   
  public static void main(String[] args)
  {

    Connection con = null;
    int iRowCount = 0;
    Statement stmt = null;

    String sDriver = "org.hsqldb.jdbcDriver";
    String sURL = "jdbc:hsqldb:hsql://localhost"; 	// uses standalone server
    String sUsername = "sa";
    String sPassword = "";

    try { 
        Class.forName( sDriver ).newInstance();
        
    } catch( Exception e ) {
    	
      System.err.println( "Failed to load driver "+sDriver);
      return;
    }

    try {

      con = DriverManager.getConnection ( sURL,
                                          sUsername,
                                          sPassword);
      System.out.println(con.getMetaData().supportsResultSetType(ResultSet.CONCUR_UPDATABLE));
      System.out.println(con.getMetaData().supportsBatchUpdates());
      System.exit(1);
//      stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, 
//                                 ResultSet.CONCUR_UPDATABLE);
    }
    catch ( Exception e)
    {
      System.err.println( "problems connecting to " + sURL + ":" );
      System.err.println( e.getMessage() );

      if( con != null)
      {
        try { con.close(); }
        catch( Exception e2 ) {
        	    	e2.printStackTrace();
        }
      }

      return;
    } // end catch

    try
    {
      stmt.executeUpdate( "DROP TABLE EMPLOYEES" );
      System.out.println( "Table EMPLOYEES was removed.");
    }
    catch ( Exception e ) { 
    	e.printStackTrace();
    }


    // create table EMPLOYEES and insert data
    try
    {
      stmt.executeUpdate( "CREATE TABLE EMPLOYEES (" +
             "ID         INTEGER      PRIMARY KEY, "   +
             "FIRSTNAME  VARCHAR (15) NOT NULL, "   +
             "SURNAME    VARCHAR (20) NOT NULL "   +
                                          ")" );

      System.out.println("Table EMPLOYEES created, now inserting...");

      for (int i = 0; i < employees.length; i++)
      {
        iRowCount +=
        stmt.executeUpdate(
          "INSERT INTO EMPLOYEES VALUES ("
           + employees[i].toSQLValue()
           + ")"
        );
      }

      System.out.println( iRowCount + " Rows inserted into EMPLOYEES.");
         
      iRowCount = 0;
         
	  // insert messages         

    try
    {
      stmt.executeUpdate( "DROP TABLE MESSAGES" );
      System.out.println( "Table MESSAGES was removed.");
    }
    catch ( Exception e ) { 
    	e.printStackTrace();
    }

      stmt.executeUpdate( "CREATE TABLE MESSAGES (" +
             "ID      INTEGER PRIMARY KEY, "   +
             "CREATED DATE NOT NULL, "   +
             "FROM    INTEGER NOT NULL, "   +
             "TO      INTEGER NOT NULL, "   +
             "TEXT    VARCHAR(100)"   +
                      ")" );

      System.out.println("Table MESSAGES created, now inserting...");

      testTransactions(con, stmt);
      

    }
    catch ( Exception e ) {
      System.err.println( e.getMessage() );
    }
    finally {
      try { 
      		stmt.close(); 
      } catch( Exception e ) {
      		e.printStackTrace();
      }
      try { 
      		con.close(); 
      } catch( Exception e ) {
      		e.printStackTrace();
      }
    } // end finally clause

	} // end main

	protected static void testTransactions(Connection con, Statement stmt) throws IOException,SQLException {
		
	  con.setAutoCommit(false);
	  
	  // inserts first two messages	
      for (int i = 0; i < TOTAL_MSGS; i++)
      {
        stmt.executeUpdate(
          "INSERT INTO MESSAGES VALUES ("
		   + newMessage(i).toSQLValue()
           + ")"
        );
      }
      
      con.commit();

      System.out.println("Rows inserted into MESSAGES.");
      
      query(stmt);
      
	}
	
	protected static void query(Statement stmt) throws IOException,SQLException {

        ResultSet result = stmt.executeQuery(
	        "SELECT TEXT " +
	        "FROM MESSAGES " +
	        "WHERE ID > 10");
	
	    result.absolute(5);

        result.updateString("TEXT", "Zmeneny text zpravy"); 
        
        result.updateRow(); 
        
	}


} // end class 
class Employee {
    
    private int id;
    private String firstname, surname;
    
    public Employee(int i, String fn, String sn) {
		id = i;
    	firstname = fn;
    	surname = sn;
    }

    public void setFirstname(String n) {
        firstname = n;
    }
    
    public String getFirstname() {
        return firstname;
    }

    public void setSurname(String n) {
        surname = n;
    }
    
    public String getSurname() {
        return surname;
    }
    
    public void setId(int a) {
        id = a;
    }
    
    public int getId() {
        return id;
    }
    
    public String toString() {
       return getId() + ", "
	       + "'" + getFirstname() + "'," 
    	   + "'" + getSurname() + "'";
    }
    
    public String toSQLValue() {
       return toString();
    }
}

class Message {
	
	private static int count = 0;
    
    private int id;
    private java.util.Date created;
    private Employee from, to;
    private String text;
    
    public Message() {
    	id = ++count;
    	created = new java.util.Date();
   	}
    
    public Message(Employee f, Employee t, String x) {
    	this();
    	from = f;
    	to = t;
    	text = x;
    }

    public void setId(int a) {
        id = a;
    }
    
    public int getId() {
        return id;
    }
    
    public java.util.Date getCreated() {
        return created;
    }
    
    public void setCreated(java.util.Date c) {
        created = c;
    }
    
    public void setFrom(Employee e) {
        from = e;
    }
    
    public Employee getFrom() {
        return from;
    }
    
    public void setTo(Employee e) {
        to = e;
    }
    
    public Employee getTo() {
        return to;
    }
    
    public void setText(String t) {
        text = t;
    }
    
    public String getText() {
        return text;
    }
    
    public String toString() {
    	return "Message "+getId()+": "
    	      +"created: "+getCreated()+", "
    	      +"from: "+getFrom()+", "
    	      +"to: "+getTo()+", "
    		  +"text: "+getText();
    }

    public String toSQLValue() {
       return getId()+", "
    	      +"'"+new java.sql.Date(getCreated().getTime())+"', "
    	      +getFrom().getId()+", "
    	      +getTo().getId()+", "
    		  +"'"+getText()+"'";
    }
}