package tomp.ucebnice.kolekce;
import tomp.ucebnice.polozky.PolozkaComparable;
import java.util.*;
abstract class TestujKolekci {
    int pocetHodnot;
    long start, time;
    void start() {
        start = System.currentTimeMillis();
    }
    void stop() {
        time = System.currentTimeMillis() - start;
    }
    public abstract void test1();
    public abstract void test2();
}
abstract class TestujSeznam extends TestujKolekci {
    List list;
    public void test1() {
        start();
        for (Iterator i = list.iterator(); i.hasNext();) {
            PolozkaComparable p
                = (PolozkaComparable)i.next();
            p.info();
        }
        stop();
        System.out.println("Prochazeni "
            +list.getClass().getName()
            +" iteratorem trvalo "+time+" ms.");
    }
    public void test2() {
        start();
        for (int i = list.size()-1; i >= 0; i--) {
            PolozkaComparable p
                = (PolozkaComparable)list.get(i);
            p.info();
        }
        stop();
        System.out.println("Prochazeni "
            +list.getClass().getName()
            +" indexem trvalo "+time+" ms.");
    }
    public void fill(int pocetHodnot) {
        this.pocetHodnot = pocetHodnot;
        for (int i = 0; i < pocetHodnot; i++) {
            list.add(new PolozkaComparable("polozka "+i, i));
        }
    }
}
class TestujZretezenySeznam extends TestujSeznam {
    TestujZretezenySeznam(int pocetHodnot) {
        list = new LinkedList();
        fill(pocetHodnot);
    }
}
class TestujArraySeznam extends TestujSeznam {
    TestujArraySeznam(int pocetHodnot) {
        list = new ArrayList();
        fill(pocetHodnot);
    }
}
class TestujMnozinu extends TestujKolekci {
    Set set;
    TestujMnozinu() {}
    TestujMnozinu(int pocetHodnot) {
        set = new HashSet();
        fill(pocetHodnot);
    }
    public void test1() {
        start();
        for (Iterator i = set.iterator(); i.hasNext();) {
            PolozkaComparable p
                = (PolozkaComparable)i.next();
            p.info();
        }
        stop();
        System.out.println("Prochazeni "
            +set.getClass().getName()
            +" iteratorem trvalo "+time+" ms.");
    }
    public void test2() {
        start();
        for (int i = 0; i < pocetHodnot; i++) {
            set.contains(
                new PolozkaComparable("polozka "+i, i));
        }
        stop();
        System.out.println("Vyhledavani v "
            +set.getClass().getName()
            +" trvalo "+time+" ms.");
    }
    public void fill(int pocetHodnot) {
        this.pocetHodnot = pocetHodnot;
        for (int i = 0; i < pocetHodnot; i++) {
            set.add(
                new PolozkaComparable("polozka "+i, i));
        }
    }
}
class TestujUsporadanouMnozinu extends TestujMnozinu {
    TestujUsporadanouMnozinu(int pocetHodnot) {
        set = new TreeSet();
        fill(pocetHodnot);
    }
}
class TestujMapu extends TestujKolekci {
    Map map;
    String[] klice;
    TestujMapu() {}
    TestujMapu(int pocetHodnot) {
        map = new HashMap();
        fill(pocetHodnot);
    }
    public void test1() {
        start();
        for (Iterator i = map.entrySet().iterator(); i.hasNext();) {
            PolozkaComparable p = (PolozkaComparable)
                ((Map.Entry)i.next()).getValue();
            p.info();
        }
        stop();
        System.out.println("Prochazeni "
            +map.getClass().getName()
            +" via entrySet() trvalo "+time+" ms.");
    }
    public void test2() {
        start();
        for (int i = 0; i < pocetHodnot; i++) {
            PolozkaComparable p = (PolozkaComparable)
                map.get(klice[i]);
            p.info();
        }
        stop();
        System.out.println("Vyhledavani v "
            +map.getClass().getName() +" trvalo "+time+" ms.");
    }
    public void fill(int pocetHodnot) {
        this.pocetHodnot = pocetHodnot;
        klice = new String[pocetHodnot];
        for (int i = 0; i < pocetHodnot; i++) {
            String key = klice[i] = "polozka "+i;
            map.put(key, new PolozkaComparable(key, i));
        }
    }
}
class TestujUsporadanouMapu extends TestujMapu {
    TestujUsporadanouMapu(int pocetHodnot) {
        map = new TreeMap();
        fill(pocetHodnot);
    }
}
public class Kolekce {
    public static void main(String[] args) {
        int pocetHodnot;
        try {
            pocetHodnot = Integer.parseInt(args[0]);
        } catch (NumberFormatException nfe) {
            pocetHodnot = 10000;
        } catch (ArrayIndexOutOfBoundsException obe) {
            pocetHodnot = 10000;
        }
        TestujKolekci[] t = {
                        new TestujArraySeznam(pocetHodnot),
                        new TestujZretezenySeznam(pocetHodnot),
                        new TestujMnozinu(pocetHodnot),
                        new TestujUsporadanouMnozinu(pocetHodnot),
                        new TestujMapu(pocetHodnot),
                        new TestujUsporadanouMapu(pocetHodnot)
                    };
        for (int i = 0; i < t.length; i++) {
            t[i].test1();
            t[i].test2();
        }
    }
}
