
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Iterator;
import java.io.PrintWriter;

public class UniversityImpl implements University {

    private SortedSet courses = new TreeSet();

    /** 
     * Tato metoda vytvo nov kurz a zaad jej do seznamu existujcch kurz. 
     * Pokud ji takov kurz existuje (tj. kurz se stejnm kdem), 
     * vygeneruje vjimku IllegalArgumentException.
     */    
    public Course createCourse(String code, String name, String description) throws InvalidCourseException {
	Course c = new CourseImpl(code,name,description);
	if (!courses.add(c)) {
	    throw new InvalidCourseException("Kurz ji existuje");
	}
	return c;
    }

    /** 
     * Tato metoda vype seznam kurz uspodan podle kd pedmtu do zadanho proudu znak.
     * Kad kurz se vype na samostatn dek, nejdve kd a pak jmno pedmtu oddlen jednou mezerou. Nap.: 
     * PB162 Programovn v jazyce Java
     * PB138 Modern znakovac jazyky a jejich aplikace 
     */
    public void listCourses(PrintWriter p) {
	
	Iterator i = courses.iterator();
	while (i.hasNext()) {
	    Course c = (Course) i.next();
	    p.print(c.getCode());
	    p.print(" ");
	    p.println(c.getName());
	}
    }

    /**
     * Metoda zjist, nachz-li se dan kurz v seznamu existujcch kurz
     */
    public boolean containsCourse(Course c) {
	return courses.contains(c);
    }

    /**
     * Metoda ze seznamu kurz odstran dan kurz. Pokud dan kurz neexistuje, vygeneruje vjimku InvalidCourseException.
     */
    public void removeCourse(Course c) throws InvalidCourseException {
	if (!courses.remove(c)) {
	    throw new InvalidCourseException("Kurz nen v seznamu kurz");
	}
    }
    
}
