package tomp.ucebnice.razeni;
import java.util.Arrays;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;
/**
   */
public class Razeni {
    static boolean zobrazovat = false;
    abstract class Sort {
        int pocetHodnot;
        long startInitTime, initTime, startSortTime, sortTime;
        Sort(int pocetHodnot) {
            this.pocetHodnot = pocetHodnot;
            startInitTime = System.currentTimeMillis();
        }
        void startSort() {
            show();
            startSortTime = System.currentTimeMillis();
            initTime = startSortTime - startInitTime;
        }
        void stop() {
            sortTime = System.currentTimeMillis() - startSortTime;
            show();
            System.out.println( "Razeni " + pocetHodnot
                                + " cisel pomoci "
                                + getClass().getName()
                                + " trvalo " + sortTime + " ms.");
        }
        abstract void show();
        void sort() {
            startSort();
        }
    }
    abstract class ArraySort extends Sort {
        int[] hodnoty;

        ArraySort(int pocetHodnot) {
            super(pocetHodnot);
            hodnoty = new int[pocetHodnot];
            for (int i = 0; i < hodnoty.length; i++) {
                hodnoty[i] = (int)Math.round(Math.random() * pocetHodnot);
            }
        }
        void swap(int i, int j) {
            int pom = hodnoty[i];
            hodnoty[i] = hodnoty[j];
            hodnoty[j] = pom;
        }
        void show() {
            if (zobrazovat) {
                for (int i = 0; i < hodnoty.length; i++) {
                    if (i > 0)
                        System.out.print(", ");
                    System.out.print(hodnoty[i]);
                }
                System.out.println();
            }
        }
    }

    class BubbleSort extends ArraySort {
        BubbleSort(int pocetHodnot) {
            super(pocetHodnot);
            System.out.println(
                "Priprava k razeni bublinovou metodou trvala "+initTime+" ms.");
        }
        void sort() {
            super.sort();
            for (int i = pocetHodnot-1; i > 0; i--)
                for (int j = 0; j < i; j++)
                    if (hodnoty[j] > hodnoty[j+1])
                        swap(j, j+1);
            stop();
        }
    }

    class SelectSort extends ArraySort {
        SelectSort(int pocetHodnot) {
            super(pocetHodnot);
            System.out.println(
                "Priprava k razeni metodou vyberu minima trvala "+initTime+" ms.");
        }
        void sort() {
            super.sort();
            for (int i = 0; i < pocetHodnot; i++) {
                int min = i;
                for (int j = i; j < pocetHodnot; j++)
                    if (hodnoty[j] < hodnoty[min])
                        min = j;
                swap(i,min);
            }
            stop();
        }
    }

    class JavaSort extends ArraySort {
        JavaSort(int pocetHodnot) {
            super(pocetHodnot);
            System.out.println(
                "Priprava k razeni metodou Arrays.sort(int[]) minima trvala "+initTime+" ms.");
        }
        void sort() {
            super.sort();
            Arrays.sort(hodnoty);
            stop();
        }
    }

    class ContainerObjectSort extends Sort {
        SortedSet hodnoty;
        ContainerObjectSort(int pocetHodnot) {
            super(pocetHodnot);
            hodnoty = new TreeSet();
            for (int i = 0; i < pocetHodnot; i++) {
                hodnoty.add(new Integer(i));
            }
            super.sort();
            System.out.println(
                "Priprava k razeni pridavanim do SortedSet trvala "
                +initTime+" ms.");
            stop();
        }
        void show() {
            if (zobrazovat) {
                Iterator i = hodnoty.iterator();
                if (i.hasNext())
                    System.out.print(i.next().toString());
                while (i.hasNext()) {
                    System.out.print(", ");
                    System.out.print(i.next().toString());
                }
                System.out.println();
            }
        }
     }

    void spust(int pocetHodnot) {
        Sort[] s = { new SelectSort(pocetHodnot),
                     new BubbleSort(pocetHodnot),
                     new JavaSort(pocetHodnot),
                     new ContainerObjectSort(pocetHodnot)
                   };
        System.out.println("--- start ---");
        for (int i = 0; i < s.length; i++) {
            s[i].sort();
        }
        System.out.println("--- stop ---");
    }
    public static void main(String args[]) {
        if (args.length == 1)
            new Razeni().spust(Integer.parseInt(args[0]));
        else
            new Razeni().spust(5000);
    }
}