package tomp.ucebnice.vyhledavani;
import java.util.Arrays;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

abstract class Search {
    int pocetHodnot;
    long startInitTime, initTime, startSortTime, sortTime;
    Search(int pocetHodnot) {
        this.pocetHodnot = pocetHodnot;
        startInitTime = System.currentTimeMillis();
    }
    abstract boolean search(int h);
    abstract void show();
}
abstract class ArraySearch extends Search {
    int[] hodnoty;

    ArraySearch(int pocetHodnot) {
        super(pocetHodnot);
        hodnoty = new int[pocetHodnot+1];
        for (int i = 0; i < hodnoty.length; i++) {
            hodnoty[i] = (int)Math.round(Math.random() * pocetHodnot);
        }
        initTime = System.currentTimeMillis() - startInitTime;
    }
    void show() {
        if (Vyhledavani.zobrazovat) {
            for (int i = 0; i < hodnoty.length; i++) {
                if (i > 0)
                    System.out.print(", ");
                System.out.print(hodnoty[i]);
            }
            System.out.println();
        }
    }
}

class SequentialSearch extends ArraySearch {
    SequentialSearch(int pocetHodnot) {
        super(pocetHodnot);
        System.out.println(
            "Priprava k sekvencnimu vyhledavani trvala "+initTime+" ms.");
    }
    boolean search(int h) {
        hodnoty[pocetHodnot] = h;
        int i = 0;
        while (h != hodnoty[i])
            i++;
        return i < pocetHodnot;
    }
}

class JavaBinarySearch extends ArraySearch {
    JavaBinarySearch(int pocetHodnot) {
        super(pocetHodnot);
        Arrays.sort(hodnoty);
        System.out.println(
            "Priprava k binarnimu vyhledavani metodou Arrays.binarySearch trvala "
            +initTime+" ms.");
    }
    boolean search(int h) {
        return Arrays.binarySearch(hodnoty, h) >= 0;
    }
}

class SortedSetSearch extends Search {
    SortedSet hodnoty;
    SortedSetSearch(int pocetHodnot) {
        super(pocetHodnot);
        hodnoty = new TreeSet();
        for (int i = 0; i < pocetHodnot; i++) {
            hodnoty.add(new Integer(i));
        }
        initTime = System.currentTimeMillis() - startInitTime;
        System.out.println(
            "Priprava k vyhledavani v SortedSet trvala "
            +initTime+" ms.");
    }
    boolean search(int h) {
        return hodnoty.contains(new Integer(h));
    }
    void show() {
        if (Vyhledavani.zobrazovat) {
            Iterator i = hodnoty.iterator();
            if (i.hasNext())
                System.out.print(i.next().toString());
            while (i.hasNext()) {
                System.out.print(", ");
                System.out.print(i.next().toString());
            }
            System.out.println();
        }
    }
 }

/**
   */
public class Vyhledavani {
    static boolean zobrazovat = false;
    void spust(int pocetHodnot) {
        Search[] s = {
                     new SortedSetSearch(pocetHodnot),
                     new SequentialSearch(pocetHodnot),
                     new JavaBinarySearch(pocetHodnot)
                   };
        System.out.println("--- start ---");
        for (int i = 0; i < s.length; i++) {
            s[i].search(pocetHodnot >>> 1);
            s[i].show();
        }
        System.out.println("--- stop ---");
    }
    public static void main(String args[]) {
        if (args.length == 1)
            new Vyhledavani().spust(Integer.parseInt(args[0]));
        else
            new Vyhledavani().spust(5000);
    }
}