package tomp.xml.util;

import java.util.*;
import java.util.regex.*;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;

public class XPP {

    protected Properties tags;

    protected boolean matched = false;    

   public static final String EOL = "\n";
   public static final char TAG = '%';
   public static final char ETAG = '|';
   public static final char BBODY = '{';
   public static final char EBODY = '}';

    static Pattern commentPattern = Pattern.compile("#(#?)\\-\\-([^\\r\\n]*)");
    // &comment($1, $2) /ge
    
    static Pattern valueOfSelectPattern = Pattern.compile("\\^([^\\^]*)(\\^)");
    // &valueOfSelect($1) /ge
    
    static Pattern emptyElementPattern = 
        Pattern.compile("\\|([\\w\\:]+)(([#\\./])(\\w+))?\\s*([^\\|]*)\\|");
    // &element($1, $3, $4, $5) /ge
    
    static Pattern totalEmptyElementPattern = 
        Pattern.compile("%%([\\w\\:]+)(([#\\./])(\\w+))?");
    // &element($1, $3, $4) /ge
    
    static Pattern elementPattern = Pattern.compile(
        "%([\\w\\:]+)(([#\\.])(\\w+))?\\s*([^\\{]*)\\{([^\\{\\}]*)\\}");
    // &element($1, $3, $4, $5, $6) /ge

    static Pattern[] escapedEntities = new Pattern[] { 
        Pattern.compile("\\\\%"), 
        Pattern.compile("\\\\\\^"), 
        Pattern.compile("\\\\\\|"),
        Pattern.compile("\\\\\\{"), 
        Pattern.compile("\\\\\\}")
        };
    static String[] replacement = new String[] {
        "&#37;", "&#94;", "&#166;",
        "&#123;", "&#124;"
        };

    static Pattern[] escapedEntities2 = new Pattern[] {
        Pattern.compile("\\&\\#37\\;"), 
        Pattern.compile("\\&\\#94\\;"), 
        Pattern.compile("\\&\\#166\\;"),
        Pattern.compile("\\&\\#123\\;"), 
        Pattern.compile("\\&\\#124\\;")
        };
    static String[] replacement2 = new String[] { 
        "%", 
        "^", 
        "|",
        "{", 
        "}"
    };
     
    public static void main(String[] params)
       throws IOException
    {
        String mode = null;
        boolean verbose = false;
        boolean fromURL = false;
        File dest = null;
        
        int pi = 0;
        while (params[pi].charAt(0) == '-') {

            if (params[pi].equalsIgnoreCase("-verbose")) {
                verbose = true;

            } else if (params[pi].equalsIgnoreCase("-url")) {
                fromURL = true;

            } else if (params[pi].equalsIgnoreCase("-mode")) {
                pi++;
                mode = params[pi];
                
            } else if (   params[pi].equalsIgnoreCase("-out") 
                       || params[pi].equalsIgnoreCase("-outfile") ) {
                pi++;
                dest = new File(params[pi]);
            }

            pi++;
        }

        if (params.length <= pi)
            throw new IllegalArgumentException(
                "Usage: XPP [-mode m][-verbose] source_file\n");

        if (fromURL) {
            new XPP().preprocess(new URL(params[pi]), mode, dest, verbose);
        } else {
            new XPP().preprocess(new File(params[pi]), mode, dest, verbose);
        }
    }


   public void preprocess(File f, String mode, File dest, boolean verbose) 
   throws IOException {
    
        String line = getFileContent(f);
        if (verbose) 
            System.err.print("Source file '"+f+"' loaded, preprocessing...");

        preprocess(line, mode, dest, verbose);
   }
   
   public void preprocess(URL u, String mode, File dest, boolean verbose) 
   throws IOException {
    
        String line = getFileContent(u);
        if (verbose) 
            System.err.print("Source from '"+u+"' downloaded, preprocessing...");

        preprocess(line, mode, dest, verbose);
   }
   
   public void preprocess(String line, String mode, File dest, boolean verbose) 
   throws IOException {

        tags = new Properties();
        if(mode != null) {
            try {
                tags.load(new FileInputStream(new File(mode+".tags.properties")));
                if (verbose) 
                    System.err.println("Mode file '"+mode+".tags.properties' loaded.");
            } catch(IOException ioe) {
                if (verbose) 
                    System.err.println("Cannot load mode file '"+mode+".tags.properties'.");
            }
        }    

//        System.out.println("Read:"+line);

         // replace all 3 entities
         for (int ei = 0; ei < escapedEntities.length; ei++) {
//            System.out.println("==="+escapedEntities[ei].pattern());
            line = escapedEntities[ei].matcher(line).replaceAll(replacement[ei]);
         }

         line = transform(line);
//         System.out.println("\nWritten:"+line);

         for (int ei = 0; ei < escapedEntities2.length; ei++) {
//            System.out.println("==="+escapedEntities2[ei].pattern());
            line = escapedEntities2[ei].matcher(line).replaceAll(replacement2[ei]);
         }
         
         if (dest == null) {
            System.out.println(line);
         } else {
            FileWriter fw = new FileWriter(dest);
            fw.write(line);
            fw.close();
         }
        if (verbose) 
            System.err.println("done");
    }
    
    protected String transform(String line) {
        
        do {
            matched = false;
       
            line = translateComment(line);
            line = translateValueOfSelect(line);
            line = translateEElement(line);
            line = translateTEElement(line);
            line = translateElement(line);
            
        } while(matched);

        return line;
    }
    
    protected String translateComment(String line) {
        Matcher m = commentPattern.matcher(line);
        if(m.find()) {
            matched = true;
            return   line.substring(0, m.start())
                   + makeComment(m.group(1), m.group(2))       
                   + line.substring(m.end());
        } else {
            return line;
        }    
    }

    protected String translateValueOfSelect(String line) {
        Matcher m = valueOfSelectPattern.matcher(line);
        if(m.find()) {
            matched = true;
            return   line.substring(0, m.start())
                   + makeValueOfSelect(m.group(1))       
                   + line.substring(m.end());
        } else {
            return line;
        }    
    }

    protected String translateEElement(String line) {
        Matcher m = emptyElementPattern.matcher(line);
        if(m.find()) {
            matched = true;
/*            
                for (int j = 1; j <= m.groupCount(); j++) {
                    System.out.println(j+"=>"+m.group(j));
                }
*/                
            return   line.substring(0, m.start())
                   + makeElement(m.group(1), m.group(3), m.group(4), m.group(5), "")
                   + translateEElement(line.substring(m.end()));
        } else {
            return line;
        }
    }        

    protected String translateTEElement(String line) {
        Matcher m = totalEmptyElementPattern.matcher(line);
        if(m.find()) {
            matched = true;
            return   line.substring(0, m.start())
                   + makeElement(m.group(1), m.group(3), m.group(4), "", "")
                   + translateTEElement(line.substring(m.end()));
        } else {
            return line;
        }
    }        

    protected String translateElement(String line) {
        Matcher m = elementPattern.matcher(line);
        if(m.find()) {
            matched = true;
            return   line.substring(0, m.start())
                   + makeElement(m.group(1), m.group(3), m.group(4), m.group(5), m.group(6))
                   + translateElement(line.substring(m.end()));
        } else {
            return line;
        }
    }        

    protected static String makeComment(String type, String content) {
    	if (type.equals("#")) {
    		return "<xsl:comment>"+content+"</xsl:comment>";
    	} else  {
    		return "<!--"+content+"-->";
    	}
    }

    protected static String makeValueOfSelect(String expr) {
    	return "<xsl:value-of select=\""
    	      +expr
    	      +"\"/>";
    }

    protected String makeElement(
        String tagname, String separator, String postfix,
        String attrs, String body) {
          
        if (attrs == null)
            attrs = "";
        String additional = null;
            
    	String tag = tags.getProperty(tagname, tagname);
    	
    	if (separator == null) {
    	    additional = "";
    	    
        } else if (separator.equals("#")) {
        		if (tag.startsWith("xsl")) {
        			additional = " name=\""+postfix+"\"";
        		} else {
        			additional = " id=\""+postfix+"\"";
        		}
        } else if (separator.equals(".")) {
        		if (tag.startsWith("xsl")) {
        			additional = " test=\""+postfix+"\"";
        		} else {
        			additional = " class=\""+postfix+"\"";
        		}
        } else if (separator.equals("/")) {
        		if (tag.equals("xsl:template")) {
        			additional = " match=\""+postfix+"\"";
        		} else {
        			additional = " select=\""+postfix+"\"";
        		}
        } else {
    		    additional = "";
    	}
    
    	if (attrs.length() > 0)
    	    attrs = " " + attrs;
    
    	if (body == null || body.length() == 0) {
    		return "<"+ tag + additional + attrs+"/>";
    	} else {
    		return "<"+ tag + additional + attrs+">"+body+"</"+tag+">";
    	}
    }

    public static String getFileContent(File f) {
        try {
            FileReader rd = new FileReader(f);
            StringWriter wr = new StringWriter();

            int oneChar;
            while ((oneChar=rd.read()) != -1)
                wr.write(oneChar);
            rd.close();
            wr.close();
            return wr.toString();
        } catch (IOException ioe) {
            return "";
        }
    }

    public static String getFileContent(URL u) {
        try {
            Reader rd = new InputStreamReader(u.openStream());
            StringWriter wr = new StringWriter();

            int oneChar;
            while ((oneChar=rd.read()) != -1)
                wr.write(oneChar);
            rd.close();
            wr.close();
            return wr.toString();
        } catch (IOException ioe) {
            return "";
        }
    }

    public static String getFileContent(String whichLines, File f) {
        if (whichLines == null || whichLines.length() == 0)
            return getFileContent(f);

        int startFromLine, numberLines;
        try {
            startFromLine = Integer.parseInt(whichLines.substring(0, whichLines.indexOf('#')));
            numberLines   = Integer.parseInt(whichLines.substring(whichLines.indexOf('#')+1));
        } catch (NumberFormatException nfe) {
            return getFileContent(f);
        }

        try {
            BufferedReader br = new BufferedReader(new FileReader(f));
            StringBuffer wr = new StringBuffer();
            String ls = System.getProperty("line.separator");

            for (String line = br.readLine(); line != null; line = br.readLine() ) {
                if (--startFromLine <= 0 && --numberLines >= 0) {
                    wr.append(line);
                    wr.append(ls);
                }
            }
            br.close();
            return wr.toString();

        } catch (IOException ioe) {
            return "";
        }
    }

}
